--[[Author:		Cyprias 
	License:	All Rights Reserved
	Contact:	Cyprias on Curse.com or WowAce.com

	Only Curse.com, Wowace.com and WoWInterface.com have permission to host this addon.
	I have not given permission for Totem Radius to be used in any addon compilation or UI pack.]]
	
-- global lookup
local _G									= _G
local pairs_ 								= pairs
local InterfaceOptionsFrame_OpenToCategory_	= InterfaceOptionsFrame_OpenToCategory
local GetCVar 								= GetCVar
local select_								= select
local UnitInParty_							= UnitInParty
local UnitInRaid_							= UnitInRaid
local GetTime_								= GetTime
local GetTotemInfo_							= GetTotemInfo
local select_ 								= select
local Minimap_								= Minimap
local CreateFrame_ 							= CreateFrame
local unpack_ 								= unpack
local GetCVar 								= GetCVar
local math_deg								= math.deg
local math_atan2							= math.atan2
local GetNumPartyMembers_					= GetNumPartyMembers
local GetNumRaidMembers_					= GetNumRaidMembers
local GetPlayerMapPosition = GetPlayerMapPosition
local GetCurrentMapContinent = GetCurrentMapContinent
local GetCurrentMapZone = GetCurrentMapZone
local SetMapToCurrentZone = SetMapToCurrentZone
local SetMapZoom = SetMapZoom
local tostring = tostring
local IsInInstance = IsInInstance
local GetSpellInfo = GetSpellInfo
local GetPlayerFacing = GetPlayerFacing
local math_sin = math.sin
local math_cos = math.cos
local rotateMinimap = GetCVar("rotateMinimap") == "1"
local MiniMapCompassRing = MiniMapCompassRing

local folder, core = ...
TR = core --global table

-- Isolate the environment
local _G = getfenv(0)
setmetatable(TR, {__index = _G})
setfenv(1, TR)
-- local
title		= "Totem Radius"
version		= GetAddOnMetadata(folder, "X-Curse-Packaged-Version") or ""
titleFull	= title.." "..version
addonDir = "Interface\\AddOns\\TotemRadius\\"

LibStub("AceAddon-3.0"):NewAddon(core, folder, "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceComm-3.0", "AceSerializer-3.0") 
L = LibStub("AceLocale-3.0"):GetLocale(folder, true)


--~ local AceGUI = LibStub("AceGUI-3.0")

commPrefixTotemLoc		= "TR_L"--New totem info
commPrefixRemoveTotem	= "TR_K"--Remove totem.

--~ Astrolabe = DongleStub("Astrolabe-0.4")

activeTotems = {}
totemIcon                = "Interface\\TargetingFrame\\UI-RaidTargetingIcon_" --use raid icons if we don't know the totem icon.
minimapSize = { -- size of minimap in yards.
	indoor = {
		[0] = 150,
		[1] = 120,
		[2] = 90,
		[3] = 60,
		[4] = 40,
		[5] = 25,
	},
	outdoor = {
		[0] = 233 + 1/3,
		[1] = 200,
		[2] = 166 + 2/3,
		[3] = 133 + 1/6,
		[4] = 100,
		[5] = 66 + 2/3,
	},
}

local regEvents = {
	"PLAYER_TOTEM_UPDATE",
	"MINIMAP_UPDATE_ZOOM",
	"COMBAT_LOG_EVENT_UNFILTERED",
	"CVAR_UPDATE",
}
db	= {}
P	= {} --db.profile

userName = ""
indoors = Minimap:GetZoom() == GetCVar( "minimapInsideZoom" ) + 0;

defaultSettings	= {
	profile = {
		totemColour= {},
		totemOpts = {},
	},
}

function core:OnInitialize()
	db = LibStub("AceDB-3.0"):New("TR_DB", defaultSettings)
	CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")
	self:RegisterChatCommand("tr", "MySlashProcessorFunc")
	
	
	local config = LibStub("AceConfig-3.0")
	local dialog = LibStub("AceConfigDialog-3.0")
	config:RegisterOptionsTable(title, CoreOptionsTable)
	coreOpts = dialog:AddToBlizOptions(title, titleFull)
	
	Minimap_ = _G.Minimap
	MiniMapCompassRing = _G.MiniMapCompassRing
	GetPlayerFacing = _G.GetPlayerFacing
end

function core:OnEnable()
	P = db.profile

	for i, event in pairs_(regEvents) do 
		self:RegisterEvent(event)
	end
	
	rotateMinimap = GetCVar("rotateMinimap") == "1"

	HookSetZoom()
	
	self:RegisterComm(commPrefixTotemLoc, "IncComm")
	self:RegisterComm(commPrefixRemoveTotem, "IncComm")

	userName = GetUnitName("player")
	GetTotemNames()
	BuildPrefixUI()
end

local function HideAllTotems()
	for GUID, data in pairs_(activeTotems) do 
		Debug("Hiding icon", GUID)
		data.icon:Hide()
	end
end

function core:OnDisable()
	HideAllTotems()
end

function GetTotemNames()
	for spellID, data in pairs(totemInfo) do 
		totemInfo[spellID].name = GetSpellInfo(spellID)
	end
end

----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset our spell exists options.	--
----------------------------------------------------------------------
	-- Shut down anything left from previous settings
	self:Disable()
	-- Enable again with the new settings
	self:Enable()
end

----------------------------------------------
function core:MySlashProcessorFunc(input)	--
-- /da function brings up the UI options.	--
----------------------------------------------
	InterfaceOptionsFrame_OpenToCategory_(coreOpts)
end

function GetCurrentPlayerPosition()
	local x, y = GetPlayerMapPosition("player");
	if ( x <= 0 and y <= 0 ) then
--~ 		if ( WorldMapVisible ) then
--~ 			-- we know there is a visible world map, so don't cause 
--~ 			-- WORLD_MAP_UPDATE events by changing map zoom
--~ 			return;
--~ 		end
		local lastCont, lastZone = GetCurrentMapContinent(), GetCurrentMapZone();
		SetMapToCurrentZone();
		x, y = GetPlayerMapPosition("player");
		if ( x <= 0 and y <= 0 ) then
			SetMapZoom(GetCurrentMapContinent());
			x, y = GetPlayerMapPosition("player");
			if ( x <= 0 and y <= 0 ) then
				-- we are in an instance or otherwise off the continent map
				return;
			end
		end
		local C, Z = GetCurrentMapContinent(), GetCurrentMapZone();
		if ( C ~= lastCont or Z ~= lastZone ) then
			SetMapZoom(lastCont, lastZone); --set map zoom back to what it was before
		end
		return C, Z, x, y;
	end
	return GetCurrentMapContinent(), GetCurrentMapZone(), x, y;
end

function GetUnitPosition( unit, noMapChange )
	local x, y = GetPlayerMapPosition(unit);
	if ( x <= 0 and y <= 0 ) then
		if ( noMapChange ) then
			-- no valid position on the current map, and we aren't allowed
			-- to change map zoom, so return
			return;
		end
		local lastCont, lastZone = GetCurrentMapContinent(), GetCurrentMapZone();
		SetMapToCurrentZone();
		x, y = GetPlayerMapPosition(unit);
		if ( x <= 0 and y <= 0 ) then
			SetMapZoom(GetCurrentMapContinent());
			x, y = GetPlayerMapPosition(unit);
			if ( x <= 0 and y <= 0 ) then
				-- we are in an instance or otherwise off the continent map
				return;
			end
		end
		local C, Z = GetCurrentMapContinent(), GetCurrentMapZone();
		if ( C ~= lastCont or Z ~= lastZone ) then
			SetMapZoom(lastCont, lastZone); -- set map zoom back to what it was before
		end
		return C, Z, x, y;
	end
	return GetCurrentMapContinent(), GetCurrentMapZone(), x, y;
end

function core:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	local timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags  = ...-- ***
	if eventType == "SPELL_SUMMON" then

		local spellID, spellName, spellSchool = select_(9,...)
		if totemInfo[spellID] and self:GetCurrentMap() then
--~ 			Debug(event, 1, srcName, dstName)
		
			local tC, tZ, tX, tY
			local precise = false
			local totemSlot = totemInfo[spellID].slot --We need the school to know what direction the totem spawned. (Fire's always 45deg, Earth 270deg, ect)
				
			if UnitIsUnit("player", srcName) then
				tC, tZ, tX = GetCurrentPlayerPosition()
				if not tC or not tX or tX == 0 then
					return
				end
				tX, tY = GetTotemCoords(totemHeadings[totemSlot])

				precise = true
			elseif P.trackOthersTotems == true and UnitInParty_(srcName) or UnitInRaid_(srcName) then
				tC, tZ, tX, tY = GetUnitPosition(srcName)
			end
--~ 			Debug(event, 2, srcName, dstName)
			if tC then
--~ 				Debug(event, 3, srcName, dstName)
				AddTotem(dstGUID, spellID, srcName, tC, tZ, tX, tY, precise, totemSlot)
				if P.broadcastTotemLocation == true and precise == true then
					BroadcastTotemLoc(dstGUID, spellID, tC, tZ, tX, tY)
				end
			end
		end
	elseif eventType == "UNIT_DIED" or eventType == "UNIT_DESTROYED" then
		if activeTotems[dstGUID] then
			RemoveTotem(dstGUID)
		end
	end
end

function core:CVAR_UPDATE(event, ...)
	local cvar, value = ...
	if cvar == "ROTATE_MINIMAP" then
		rotateMinimap = value == "1"
	end
end

local GetMapInfo = GetMapInfo
function PlaceIconOnMinimap( icon, x, y )
	local tX, tY = CoordsToYardCoords(x, y)
	
	icon.tX = tX--yard coords
	icon.tY = tY

	local found, width, height = core:GetZoneSize()
	if found then
		icon.mW = width
		icon.mH = height
		local found, currentMap = core:GetCurrentMap()
		if found then
			icon.map = currentMap
			
			icon.updateTimer = 0.05
			icon:Show()
		end
	end
--~ 	iconOnUpdate(icon, 1)
end

----------------------------------------------------------
function RemoveCastersOldTotems(caster, totemSlot)		--
-- Remove last known totem for a caster's totem slot.	--
----------------------------------------------------------
	for totemGUID, data in pairs(activeTotems) do 
		if data.caster == caster and data.totemSlot == totemSlot then
--~ 			Debug("RemoveCastersOldTotems 1", "Removing", caster, totemSlot)
			RemoveTotem(totemGUID)
			return
		end
	end
end

--[[
local function iconOnHide(self)
	self:SetParent(nil)
end
]]

local spellSlotPoint = {
	"BOTTOMRIGHT",
	"BOTTOMLEFT",
	"TOPLEFT",
	"TOPRIGHT",
}


--------------------------------------------------------------------------------------
function AddTotem(totemGUID, spellID, caster, tC, tZ, tX, tY, precise, totemSlot)	--
-- Add a totem to our table then show it on the minimap.							--
--------------------------------------------------------------------------------------
--~ 	Debug("AddTotem 1", caster, totemGUID)
	if not totemInfo[spellID] then
		return
	end
	RemoveCastersOldTotems(caster, totemSlot)
	
	local totemName = totemInfo[spellID].name or GetSpellInfo(spellID)
	if P.totemOpts[totemName] and P.totemOpts[totemName].shown == false then
--~ 		Debug("AddTotem","User doesnt want "..tostring(totemName).." shown.")
		return
	end	
	--
	
	local radius = minimapSize[indoors and "indoor" or "outdoor"][Minimap_:GetZoom()]
	local minimapWidth = Minimap_:GetWidth()
	
	local range = totemInfo[spellID].range or 0
	if precise == false and range > spawnDistance then
		range = range - spawnDistance --We don't know the precise coords of the totem. So we remove 3 yards from the totem's range to make sure we'll be inside it. 
	end
	
	local size = (minimapWidth * range/radius)
	if not activeTotems[totemGUID] then
		local totemSlot = totemSlot or totemInfo[spellID].slot or 1
		
		activeTotems[totemGUID] = {
			icon	= CreateFrame_("Button",nil,Minimap_),
			caster	= caster,
			coords	= {tC, tZ, tX, tY},
			range	= range,
			precise	= precise,
			totemSlot = totemSlot,
			death = GetTime_() + totemInfo[spellID].duration,--Remove the icon at this time, just in case UNIT_DEAD doesn't see it.
			spellID = spellID,
			name = totemName,
		}
--~ 		activeTotems[totemGUID].icon:SetScript("OnHide", iconOnHide)
		
		activeTotems[totemGUID].icon.totem = activeTotems[totemGUID] --Add our totem table to the icon frame so it can be used in the OnUpdate script.
		activeTotems[totemGUID].icon.GUID = totemGUID
		activeTotems[totemGUID].icon.spellID = spellID
		
--~ 		Debug("AddTotem 2", caster, totemGUID)
		SetOnUpdateScript(activeTotems[totemGUID].icon)
		
		activeTotems[totemGUID].icon:SetWidth(P.iconSize)
		activeTotems[totemGUID].icon:SetHeight(P.iconSize)
		
		
		activeTotems[totemGUID].icon.spellTexture = activeTotems[totemGUID].icon:CreateTexture(nil,"ARTWORK")
		activeTotems[totemGUID].icon.spellTexture:SetTexture(totemInfo[spellID].texture or totemIcon..totemSlot)
		if precise == true then
			activeTotems[totemGUID].icon.spellTexture:SetAllPoints(activeTotems[totemGUID].icon)
			
		else
			activeTotems[totemGUID].icon.spellTexture:SetWidth(P.iconSize)
			activeTotems[totemGUID].icon.spellTexture:SetHeight(P.iconSize)
		
			activeTotems[totemGUID].icon.spellTexture:SetPoint(spellSlotPoint[totemSlot], activeTotems[totemGUID].icon, "CENTER")
--~ 			Debug("AddTotem 3", totemSlot, spellSlotPoint[totemSlot])
		end
		
		if range > 0 then
			activeTotems[totemGUID].icon.outlineTexture = activeTotems[totemGUID].icon:CreateTexture(nil,"low")
			activeTotems[totemGUID].icon.outlineTexture:SetTexture(addonDir.."texture\\Outline")
			activeTotems[totemGUID].icon.outlineTexture:SetPoint("CENTER", activeTotems[totemGUID].icon);
			activeTotems[totemGUID].icon.outlineTexture:SetAlpha(1)
			
			if P.colourRingBySchool == true then
				activeTotems[totemGUID].icon.outlineTexture:SetVertexColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, 1);
			else
				activeTotems[totemGUID].icon.outlineTexture:SetVertexColor(P.insideRingColour.r, P.insideRingColour.g, P.insideRingColour.b, 1); --first drop, colour it green.
			end
		end
	elseif precise == true then
		--We're already tacking the totem but another user has sent us more accurate coords.
		Debug("AddTotem", tostring(caster).." has sent us more accurate coords for "..tostring(totemInfo[spellID].name))
		activeTotems[totemGUID].coords = {tC, tZ, tX, tY}
		activeTotems[totemGUID].range = range
	end
	local icon = activeTotems[totemGUID].icon

	if range > 0 then
		icon.outlineTexture:SetWidth(size) -- Set these to whatever height/width is needed 
		icon.outlineTexture:SetHeight(size) -- for your Texture
	end
--~ 	icon:SetScript("OnUpdate", iconOnUpdate)

	if tC == GetCurrentMapContinent() and tZ == GetCurrentMapZone() then
		if core:GetCurrentMap() then
			PlaceIconOnMinimap(icon, tX, tY)
			icon:Show()
			
			if P.AVRdisplay == true then
				AddTotemToAVR(totemGUID, tX, tY, totemSlot, spellID, range, precise)
			end
		end
	end
	--[[
	local result = Astrolabe:PlaceIconOnMinimap( activeTotems[totemGUID].icon, tC, tZ, tX, tY )
	]]
	
end

function HookSetZoom()--	/run TR.HookSetZoom()
	if not core:IsHooked(Minimap_, "SetZoom") then
		core:SecureHook(Minimap_, "SetZoom", "Minimap_SetZoom")
	end
end

--[===[@debug@
function DropFakeTotem()--	/run TR.DropFakeTotem()
	local totems = {}
	for spellID in pairs(core.totemInfo) do 
		table.insert(totems, #totems+1, spellID)
	end

	local totemID = totems[math.random(#totems)]
	Debug("DropFakeTotem 1", totemID, core.totemInfo[totemID].slot, spellSlotPoint[core.totemInfo[totemID].slot])
	
	local found, currentMap = core:GetCurrentMap()
	if found then
		local tC, tZ = GetCurrentMapContinent(), GetCurrentMapZone();
		local h = math.random(4)
--~ 		local tX, tY = GetTotemCoords(totemHeadings[h])
		local tX, tY = GetPlayerMapPosition("player")
		
		AddTotem(math.random(1000), totemID, UnitName("player"), tC, tZ, tX, tY, false, h)
	end
end
--	/run print(GetMouseFocus():GetName())
--@end-debug@]===]

function core:IncComm(prefix, message, distribution, sender)
	if sender ~= userName then
		if prefix == commPrefixTotemLoc then
			if P.trackOthersTotems == true then
				local success, totemGUID, spellID, tC, tZ, tX, tY = core:Deserialize(message)
		--~ 		Debug("IncComm",sender.." dropped a totem!")
				if success then
					AddTotem(totemGUID, spellID, sender, tC, tZ, tX, tY, true)
				end
			end
		elseif prefix == commPrefixRemoveTotem then
			local success, totemGUID = core:Deserialize(message)
			if success then
				if activeTotems[totemGUID] and activeTotems[totemGUID].caster == sender then
--~ 					Debug("IncComm","Removing "..sender.."'s totem.")
					RemoveTotem(totemGUID)
				end
			end
		end
	end
end

----------------------------------------------------------------------------------------------------------
function BroadcastTotemLoc(totemGUID, spellID, tC, tZ, tX, tY)											--
-- Send the precise coords of our totems so others can update the locs of our totems on their minimap.	--
----------------------------------------------------------------------------------------------------------
	if GetNumPartyMembers_() > 0 then
		local str = core:Serialize(totemGUID, spellID, tC, tZ, tX, tY)
	--~ 	Debug("BroadcastTotemLoc", str, str:len())
--~ 		core:SendCommMessage(commPrefixTotemLoc, str, "GUILD", nil, "ALERT")

		local iActive, iType = IsInInstance()
		if iActive and iType == "pvp" then
			core:SendCommMessage(commPrefixTotemLoc, str, "BATTLEGROUND", nil, "ALERT")
		else
			core:SendCommMessage(commPrefixTotemLoc, str, GetNumRaidMembers_() > 1 and "raid" or "party", nil, "ALERT")
		end
	end
end

----------------------------------------------------------------------------------
function BroadcastRemoveTotem(totemGUID)										--
-- Sends the totemGUID to group for others to remove if from their minimap.		--
-- This is needed cause UNIT_DEAD doesn't always fire if we're out of range.	--
----------------------------------------------------------------------------------
	if GetNumPartyMembers_() > 0 then
		local str = core:Serialize(totemGUID)
--~ 		core:SendCommMessage(commPrefixRemoveTotem, str, "GUILD", nil, "ALERT")

		local iActive, iType = IsInInstance()
		if iActive and iType == "pvp" then
			--We're in a battlefield. 
			core:SendCommMessage(commPrefixRemoveTotem, str, "BATTLEGROUND", nil, "BULK")
		else
			core:SendCommMessage(commPrefixRemoveTotem, str, GetNumRaidMembers_() > 1 and "raid" or "party", nil, "BULK")
		end
	end
end


function RemoveTotem(totemGUID)
	if activeTotems[totemGUID] then
		RemoveTotemFromAVR(totemGUID)
		
		activeTotems[totemGUID].icon:Hide()
		activeTotems[totemGUID].icon:SetParent(nil)
		activeTotems[totemGUID].icon = nil
		activeTotems[totemGUID] = nil
	end
end

function CompensateForMinimapRotate(xDist, yDist)
	if GetPlayerFacing then
		facing = GetPlayerFacing()
	else
		facing = -MiniMapCompassRing:GetFacing()
	end

	local sin = math_sin(facing)
	local cos = math_cos(facing)

	local dx, dy = xDist, yDist
	xDist = dx*cos - dy*sin
	yDist = dx*sin + dy*cos
	return xDist, yDist
end

function SetOnUpdateScript(frame)
	frame.lastUpdate = 0
--~ 	frame.lastDist = 0
	frame:SetScript("OnUpdate", function(self, elapsed)
		self.lastUpdate = self.lastUpdate + elapsed
--~ 		if (not self.onEdge and self.lastUpdate > 0.05) or (self.onEdge and self.lastUpdate > 1) then
		if self.lastUpdate > self.updateTimer  then

			self.lastUpdate = 0
			local totemGUID = self.GUID
--~ 			Debug("OnUpdate", totemInfo[self.spellID].name, self.totem.death - GetTime_())
			local found, currentMap = core:GetCurrentMap()
			
			if not found then
				Debug("OnUpdate", "Can't find current map!")
				RemoveTotem(totemGUID)
				return
			end
			
			if self.map ~= currentMap then
				--We've changed zones. Lets set the icon as transparent and set it to update once a second for now.
				self:SetAlpha(0)
				self.updateTimer = 1
				return
			else
				self:SetAlpha(1)
			end
			
			if GetTime_() >= self.totem.death then
				--Totem should have died due to old age by now. Remove it.
				RemoveTotem(totemGUID)
				return
			end

			local map = Minimap
--~ 			local minimapZoom = map:GetZoom()
		
			local tX, tY = self.tX, self.tY

			local sX, sY = GetPlayerMapPosition("player")
		
			local sX = self.mW * sX
			local sY = self.mH * sY

			local dist, xDist, yDist = ComputeDistance(sX, sY, tX, tY)
			
			local facing
			if rotateMinimap then
				xDist, yDist = CompensateForMinimapRotate(xDist, yDist)
			end

			local inside = false

			local totemSlot = self.totem.totemSlot or 1
			if dist > P.minimapIconRange then
				RemoveTotem(totemGUID)
				return
			end
			
			if dist < self.totem.range then --hasBuff and 
				if self.totem.range > 0 then
					if P.colourRingBySchool == false then
						self.outlineTexture:SetVertexColor(P.insideRingColour.r, P.insideRingColour.g, P.insideRingColour.b, P.insideRingAlpha);
					end
					inside = true --for AVR
				end
			else
				if self.totem.range > 0 then
					if dist > self.totem.range then
						if P.colourRingBySchool == false then
							self.outlineTexture:SetVertexColor(P.outsideRingColour.r, P.outsideRingColour.g, P.outsideRingColour.b, P.outsideRingAlpha);
						end
						
						inside = false
					else
						if P.colourRingBySchool == false then
							self.outlineTexture:SetVertexColor(P.insideRingColour.r, P.insideRingColour.g, P.insideRingColour.b, P.insideRingAlpha);
						end
						
						inside = true
					end
				end
			end
			
			local onEdge, xDist, xScale, yDist, yScale = GetMinimapCoordOffset(self, dist, xDist, yDist)
			
			if AVR and totemMeshes[totemGUID] then
				
				if dist > P.avrShowRange then
					for name, mesh in pairs(totemMeshes[totemGUID]) do 
--~ 						mesh.visible = false
						totemMeshes[totemGUID][name].visible = false
					end
				else
					for name, mesh in pairs(totemMeshes[totemGUID]) do 
--~ 						mesh.visible = true
						totemMeshes[totemGUID][name].visible = true
					end
				end
			
				if P.AVRdisplay == true then
					local meshes = totemMeshes[totemGUID]
					if inside == true then
						if meshes.rangeRing then
							if P.avrColourRingBySchool == true then
								meshes.rangeRing:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, P.avrInsideRingAlpha)
							else
								meshes.rangeRing:SetColor(P.insideRingColour.r, P.insideRingColour.g, P.insideRingColour.b, P.avrInsideRingAlpha)
							end
						end

						if meshes.colourRing then
							meshes.colourRing:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, P.avrInsideRingAlpha)
						end
					else
						if meshes.rangeRing then
							if P.avrColourRingBySchool == true then
								meshes.rangeRing:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, P.avrOutsideRingAlpha)
							else
								meshes.rangeRing:SetColor(P.outsideRingColour.r, P.outsideRingColour.g, P.outsideRingColour.b, P.avrOutsideRingAlpha)
							end
						end
						
						if meshes.colourRing then
							meshes.colourRing:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, P.avrOutsideRingAlpha)
						end
					end

					--firenova ring for fire totems.
					if P.avrFireNovaRing == true and meshes.firenovaRangeRing then
						if dist < fireTotemRange[totemGUID] then
							if P.avrColourRingBySchool == true then
								meshes.firenovaRangeRing:SetColor(P.avrFireNovaColour.r, P.avrFireNovaColour.g, P.avrFireNovaColour.b, P.avrInsideRingAlpha)
							else
								meshes.firenovaRangeRing:SetColor(P.insideRingColour.r, P.insideRingColour.g, P.insideRingColour.b, P.avrInsideRingAlpha)
							end
							
							if meshes.firenovaColourRing then
								meshes.firenovaColourRing:SetColor(P.avrFireNovaColour.r, P.avrFireNovaColour.g, P.avrFireNovaColour.b, P.avrInsideRingAlpha)
							end
						else
							if P.avrColourRingBySchool == true then
								meshes.firenovaRangeRing:SetColor(P.avrFireNovaColour.r, P.avrFireNovaColour.g, P.avrFireNovaColour.b, P.avrOutsideRingAlpha)
							else
								meshes.firenovaRangeRing:SetColor(P.outsideRingColour.r, P.outsideRingColour.g, P.outsideRingColour.b, P.avrOutsideRingAlpha)
							end
							
							if meshes.firenovaColourRing then
								meshes.firenovaColourRing:SetColor(P.avrFireNovaColour.r, P.avrFireNovaColour.g, P.avrFireNovaColour.b, P.avrOutsideRingAlpha)
							end
						end
					end					
				end
			end
			
			
			--AVR rings
			--[[
			if scene[self.GUID] then
				if dist > P.avrShowRange then
					scene[self.GUID].visible = false
				else
					scene[self.GUID].visible = true
					
					if P.AVRdisplay == true then
						if inside == true then
							if avrRangeRing[self.GUID] then
								avrRangeRing[self.GUID]:SetColor(P.insideRingColour.r, P.insideRingColour.g, P.insideRingColour.b, P.avrInsideRingAlpha)
							end
							if avrTotemRing[self.GUID] then
								avrTotemRing[self.GUID]:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, P.avrInsideRingAlpha)
							end
	
						else
							if avrRangeRing[self.GUID] then
								avrRangeRing[self.GUID]:SetColor(P.outsideRingColour.r, P.outsideRingColour.g, P.outsideRingColour.b, P.avrOutsideRingAlpha)
							end
							
							if avrTotemRing[self.GUID] then
								avrTotemRing[self.GUID]:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, P.avrOutsideRingAlpha)
							end
						end
						
						
						--firenova ring for fire totems.
						if P.avrFireNovaRing == true and avrFireNovaRangeRing[self.GUID] then
							if dist < avrFireNovaRangeRing[self.GUID].range then
								avrFireNovaRangeRing[self.GUID].mesh:SetColor(P.insideRingColour.r, P.insideRingColour.g, P.insideRingColour.b, P.avrInsideRingAlpha)
								
								if avrFireNovaColourRing[self.GUID] then
									avrFireNovaColourRing[self.GUID]:SetColor(P.avrFireNovaColour.r, P.avrFireNovaColour.g, P.avrFireNovaColour.b, P.avrInsideRingAlpha)
								end
								
							else
								avrFireNovaRangeRing[self.GUID].mesh:SetColor(P.outsideRingColour.r, P.outsideRingColour.g, P.outsideRingColour.b, P.avrOutsideRingAlpha)
								
								if avrFireNovaColourRing[self.GUID] then
									avrFireNovaColourRing[self.GUID]:SetColor(P.avrFireNovaColour.r, P.avrFireNovaColour.g, P.avrFireNovaColour.b, P.avrOutsideRingAlpha)
								end
							end
						end
						
					end
				end
			end
			]]

			if onEdge then
				if self.totem.range > 0 then
					self.outlineTexture:SetAlpha(0)
				end
				self.updateTimer = 0.25 --Update the position 4x a second.
			else
				if self.totem.range > 0 then

					if inside == true then
						self.outlineTexture:SetAlpha(P.insideRingAlpha)
					else
						self.outlineTexture:SetAlpha(P.outsideRingAlpha)
					end
					
					
--~ 					self.outlineTexture:SetAlpha(1)

					
				end
				self.updateTimer = 0.05 --Update the position 20x a second. This appears fluid above 20FPS.
			end
			
			self:ClearAllPoints();
			self:SetPoint("CENTER", map, "CENTER", xDist/xScale, -yDist/yScale);
			
		end
	end)
end
--	/run TR.DropFakeTotem()

local lastZoom = 0
local ignoreZoom = false


function core:MINIMAP_UPDATE_ZOOM(event, ...)
--~ 	Debug(event, "a", radius, minimapWidth)
	if ignoreZoom == true then --Keeps us from entering a endless loop due to SetZoom(zoom>0...) later on.
		return
	end
--~ 	Debug(event, "b", radius, minimapWidth)
	local zoom = Minimap:GetZoom();
	if zoom ~= lastZoom then --Keeps us from processing the same zoom info repeatedly due to SetZome(zoom) later on. 
--~ 		Debug(event, "c", radius, minimapWidth)
		lastZoom = zoom
		if ( GetCVar( "minimapZoom" ) == GetCVar( "minimapInsideZoom" ) ) then -- Indeterminate case
			ignoreZoom = true
 			Minimap:SetZoom( zoom > 0 and zoom - 1 or zoom + 1 ); -- Any change to make the cvars unequal
			ignoreZoom = false
		end
		indoors = Minimap:GetZoom() == GetCVar( "minimapInsideZoom" ) + 0;
		Minimap:SetZoom(zoom);

		local range, size
		local radius = minimapSize[indoors and "indoor" or "outdoor"][Minimap_:GetZoom()]
		
		local minimapWidth = Minimap_:GetWidth()
		
--~ 		Debug(event, "d", radius, minimapWidth)
		
--~ 		for name, data in pairs_(myTotems) do 
		for GUID, data in pairs_(activeTotems) do 
			size = (minimapWidth * data.range/radius)
			
--~ 			Debug(event, indoors, radius, minimapWidth, size)
			if data.range > 0 then
				data.icon.outlineTexture:SetWidth(size) -- Set these to whatever height/width is needed 
				data.icon.outlineTexture:SetHeight(size) -- for your Texture
			end
--~ 			Debug(event, "e", GUID, size)
		end
	end
end


function core:PLAYER_TOTEM_UPDATE(event, totemSlot)
	local haveTotem, totemName, startTime, duration = GetTotemInfo(totemSlot)
--~ 	Debug(event, totemSlot, haveTotem, totemName, startTime, duration)
	
	if startTime == 0 then
		for GUID, data in pairs_(activeTotems) do 
			if data.caster == userName and data.totemSlot == totemSlot then
--~ 				Debug(event,totemInfo[data.spellID].name)
				RemoveTotem(GUID)
				if P.broadcastTotemLocation == true then
					BroadcastRemoveTotem(GUID)
				end
				break
			end
		end
	end
end

function core:Minimap_SetZoom(self, ...)
--~ 	Debug("SetZoom", self:GetName(), ...)
	core:MINIMAP_UPDATE_ZOOM("MINIMAP_UPDATE_ZOOM")
end

function MapTest()--	/run TR.MapTest()
	local mapFile = GetMapInfo()
	Debug("MapTest", tostring(mapFile), tostring(core.zone_data[mapFile]))
end


--	/run TR.DropFakeTotem()
local f = CreateFrame("frame")
f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:SetScript("OnEvent", function(self, event)
--~ 	Debug(event)
	HideAllTotems()
end)
